#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <signal.h>
#include <sys/types.h>
#include <asm/types.h> /* __u32 */
#include <sys/time.h>
#include <sys/wait.h>
#include <sys/socket.h>
#include "forms.h"
#include "xinterface.h"
#include "mdsp.h"

#define BUFSIZE 4096
#define CMDSIZE 1024
#define MDSPTOOL "mdsptool" /* Must be in $PATH */
#define MAX_MDSP 16

/* from main.c */
extern FD_xmdsptool *xmd;
extern FL_OBJECT    *textbox;


/* Create command subprocess, redirect output */
int subprocess(char *cmd, int *out) /* returns PID of subprocess */
{
 char *argv[16],*p;
 int i,pid;
 for(i=0,p=cmd;;)
  {
   argv[i++]=p;
   p=strchr(p, ' ');
   if(p==NULL)break;
   *p++=0;
  }
 argv[i] = NULL;
 if((pid=fork())==0)
  { /* this is the child process */
   dup2(out[1],1); /* let out[1] be child processes stdout */
   dup2(1,2); /* force stderr to the pipe */
   execvp(argv[0], argv);
   perror(argv[0]);
   exit(1);
  }
 return pid;
}

/* Writes output of subprocess to textbox and retuns char buffer of current */
/* output */
char *mdsptool(int n,char *args)
{
 const char *command=MDSPTOOL;
 const char *cr="\n";
 char com[CMDSIZE];
 static char buf[BUFSIZE];
 size_t bytes=0;
 const struct timeval tc={1,0};
 struct timeval tv=tc;
 int fd=-1;
 int sfd[2];
 int pid;
 int result=0;
 int pstatus=0;
 int timeout;
 fd_set fds;
 
 sprintf(com,"%s -%d %s",command,n,args);
 if(socketpair(AF_UNIX,SOCK_STREAM,0,sfd)<0)
  {
   return NULL;
  }
#ifdef DEBUG
 fl_addto_browser_chars(textbox, "Executing command: ");
 fl_addto_browser_chars(textbox, com);
 fl_addto_browser_chars(textbox, cr);
#endif
 pid=subprocess(com,sfd);
 fd=sfd[0];
 if(pid<=0)
  {
   fl_show_alert("Fatal!","Cannot create subprocess.",NULL,1);
   if(fd>=0) { close(sfd[0]); close(sfd[1]); }
   return NULL;
  }
 FD_ZERO(&fds);
 FD_SET(fd,&fds);
 for(;;)
  {
   for(timeout=0;timeout<5&&result==0;timeout++)
    { /* Wait for data from subprocess */
     result=select(FD_SETSIZE,&fds,NULL,&fds,&tv);
     if((pstatus=waitpid(pid,NULL,WNOHANG))!=0) break;
    }
   if(result>0) /* Data arrived */
    {
     bytes=read(fd,buf,BUFSIZE-1);
     if(bytes<=0) { close(sfd[0]); close(sfd[1]); return NULL; }
     buf[bytes]=0;
     fl_addto_browser_chars(textbox, buf); /* Already has newline */
    }
   if(pstatus!=0) break; /* Process terminated */
   if(timeout==5) /* Timeout occured, kill process. */
    {
     kill(pid,SIGHUP);
     fl_show_alert("Command timed out",
                   "DSP is not responding to your command",
                   "(Maybe it's dead?)",1);
     if(fd>=0) { close(sfd[0]); close(sfd[1]); }
     return NULL;
    }
  }
 if(fd>=0) { close(sfd[0]); close(sfd[1]); }
 return buf;
}

char *mdspcmd2(int n,char *s1, char *s2)
{
 char cmd[CMDSIZE];
 strncpy(cmd,s1,CMDSIZE);
 strncat(cmd," ",CMDSIZE);
 strncat(cmd,s2,CMDSIZE);
 return mdsptool(n,cmd);
}

int hexparse(char *s,__u32 *val) 
{
 unsigned long l;
 int result;
 result=sscanf(s,"%lx",&l);
 *val=(__u32)l;
 return result!=1;
}

int ishex(char *s)
{
 unsigned long dummy;
 return sscanf(s,"%lx",&dummy);
}

/* Global */
static FL_FORM *fsel=NULL;

/* A struct for write register command callbacks */
struct fsel_struct
{
 FL_OBJECT *accept,*cancel,*input,*button[32];
 char *action;
 char **flags;
 int devno;
 unsigned long val;
 char cval[128];
};

void flag_selection_update_buttons(struct fsel_struct *fs)
{
 int i;
 unsigned long val;
 char *c=(char *)fl_get_input(fs->input);
 if(sscanf(c,"0x%lx",&val)==1)
  {
   fs->val=val;
   for(i=0;i<32;i++)
     if(val&1L<<i) fl_set_button(fs->button[i],1);
  }
}

void flag_selection_update_input(struct fsel_struct *fs)
{
 int i;
 unsigned long val;
 for(i=0,val=0;i<32;i++)
  {
   if(fl_get_button(fs->button[i])!=0)
     val|=1<<i;
   else
     val&=~(1<<i);
  }
 sprintf(fs->cval,"0x%lx",val);
 fl_set_input(fs->input,fs->cval);
}

void flag_selection_callback(FL_OBJECT *o, void *v)
{
 struct fsel_struct *fs=(struct fsel_struct *)v;
 if(o==fs->accept)
  {
   mdspcmd2(fs->devno,fs->action,fs->cval);
   fl_hide_form(fsel); fl_free_form(fsel);
   fl_activate_form(xmd->xmdsptool);
  }
 else if(o==fs->cancel)
  {
   fl_hide_form(fsel); fl_free_form(fsel);
   fl_activate_form(xmd->xmdsptool);
  }
 else if(o==fs->input)
  {
   flag_selection_update_buttons(fs);
  }
 else /* Bit Buttons */
  {
   flag_selection_update_input(fs);
  }
}

#define CHOICE_WIDTH 350
#define CHOICE_HEIGHT 280

int flag_selection(char *title,struct fsel_struct *fs)
{
 int i;
 sprintf(fs->cval,"0x%lx",(unsigned long)fs->val);
 fsel = fl_bgn_form(FL_UP_BOX,CHOICE_WIDTH,CHOICE_HEIGHT);
 for(i=0;i<32;i++)
  {
   int x=(((CHOICE_WIDTH-10)/4)*(i%4))+5;
   int y=((CHOICE_HEIGHT/46)*(i-(i%4)))+5;
   static char btext[32][32];
   sprintf(btext[i],fs->flags[i]!=NULL?fs->flags[i]:"Bit %d",i);
   fs->button[i]=fl_add_checkbutton(FL_PUSH_BUTTON,x,y,80,20,btext[i]);
   fl_set_object_boxtype(fs->button[i],FL_SHADOW_BOX);
   if(fs->val&1L<<i) fl_set_button(fs->button[i], 1);
   fl_set_object_color(fs->button[i],FL_TOP_BCOL,FL_BLACK);
  }
 fs->input=fl_add_input(FL_NORMAL_INPUT,CHOICE_WIDTH/2-65,CHOICE_HEIGHT-82,
                        130,30,"Input");
 fl_set_input(fs->input,fs->cval);
 fs->accept=fl_add_button(FL_NORMAL_BUTTON,5,CHOICE_HEIGHT-45,
                          110,40,"Accept");
 fl_set_button_shortcut(fs->accept,"Aa",1);
 fs->cancel=fl_add_button(FL_NORMAL_BUTTON,CHOICE_WIDTH-115,CHOICE_HEIGHT-45,
                          110,40,"Cancel");
 fl_set_button_shortcut(fs->cancel,"^[c^c",1);
 fl_set_form_callback(fsel,flag_selection_callback,fs);
 fl_end_form();
 fl_show_form(fsel,FL_PLACE_MOUSE,0,title);
 fl_deactivate_form(xmd->xmdsptool); /* deactivate main form */
                                     /* while showing popup */
 return 0;
}

int set_register(int n,char *cmd)
{
 static char *hstr_flags[32]=
  { "TRDY","HTRQ","HRRQ","HF3","HF4","HF5", "HINT","HREQ", };
 static char *hctr_flags[32]=
  { NULL,"TREQ","RREQ","HF0","HF1","HF2","DMAE","SFT","HTF0",
    "HTF1",NULL,"HRF0","HRF1",NULL,"HS0","HS1","HS2",
    NULL,NULL,"TWSD",}; 
 static char *pci_flags[32]=
  { "IO","MEMORY","MASTER","SPECIAL","INVALIDT","PALETTE","PARITY","WAIT",
    "WAIT","SERR","FASTBCK",NULL,NULL,NULL,NULL,NULL,
    NULL,NULL,NULL,NULL,NULL,"S_66MHZ","S_UDF","S_FASTB",
    "S_PRTY","S_MEDI","S_SLOW","S_STABRT","S_RTABRT","S_RMABRT","S_SSERR",
    "S_DPARTY",};
 static char *hcvr_flags[32]=
  { "HC","HV0","HV1","HV2","HV3","HV4","HV5","HV6","HNMI", };
 char *cval,*title;
 __u32 val;
 static struct fsel_struct fs;
 fs.action=cmd;
 fs.devno=n;
 switch(cmd[1])
  {
   case 'S': 
     cval=mdsptool(n,"-s");
     fs.flags=(char **)hstr_flags;
     title="Please enter value for HSTR";
     break;
   case 'C':
     cval=mdsptool(n,"-c");
     fs.flags=(char **)hctr_flags;
     title="Please enter value for HCTR";
     break;
   case 'I':
     cval=mdsptool(n,"-i");
     fs.flags=(char **)hcvr_flags;
     title="Please enter value for HCVR";
     break;
   case 'P':
     cval=mdsptool(n,"-p");
     fs.flags=(char **)pci_flags;
     title="Please enter value for PCI (bad values may hang PCI transfers)";
     break;
   default: return 0;
  }
 if(!hexparse(cval,&val))
  {
   fs.val=(unsigned long)val;
   sprintf(fs.cval,"0x%x",val);
   flag_selection(title,&fs);
  }
 else
  {
   fl_show_alert("Command failed", "mdsptool cannot determine status",
                "Please try again",1);
   return 1;
  }
 return 0;
}

char *upload(int n)
{
 char *fname;
        /* message   dir  pattern  default */
 fname=(char *)fl_show_fselector("Please select file for Upload",NULL,
                                 "*.pci", NULL);
 if(!fname||*fname==0) return NULL;
 return mdspcmd2(n,"-f",fname);
}

char *mdsptool_p(int n,char *cmd)
{
 char *input;
 const char *enterhex="Please enter hex number (like 0x0f)";
 unsigned int dummy,ok=0;
 do
  {
   input=(char *)fl_show_input(enterhex,"0x");
   if(input==NULL) return NULL; /* User hit Cancel */
   if(sscanf(input,"%x",&dummy)!=1)
    {
     fl_show_alert("Not a Hex number:", input,
            "Please try again",1);
    }
   else ok=1;
  }
 while(!ok);
 return mdspcmd2(n,cmd,input);
}

int verify_devices(FD_xmdsptool *x)
{
 int i,found;
 for(i=1,found=0;i<=MAX_MDSP;i++)
  {
   int fd;
   char *name=(char *)fl_get_choice_item_text(x->setdevice,i);
   if((fd=open(name,O_RDONLY))<0)
    {
     fl_set_choice_item_mode(x->setdevice,i,FL_PUP_INACTIVE);
    }
   else
    {
     close(fd);
     found++;
    }
  }
 return(found);
}

/* Universal Callback */
void callback(FL_OBJECT *o, long l)
{
 static int devno=1; /* Default DSP Board No. */
 switch(l)
  {
   case 0: exit(0);
   case 1: /* Device */ 
           devno=fl_get_choice(o); break;
   case 2: /* Upload */
           upload(devno);
           break;
   case 3: /* Stat All */
           mdsptool(devno,"-a");
           break;
   case 4: /* Print Control */
           mdsptool(devno,"-v -c");
           break;
   case 5: /* Set Control */
           set_register(devno,"-C");
           break;
   case 6: /* Print Status */
           mdsptool(devno,"-v -s"); break;
   case 7: /* Set Status */
           set_register(devno,"-S");
           break;
   case 8: /* Print Vector */
           mdsptool(devno,"-v -i"); break;
   case 9: /* Set Vector */
           set_register(devno,"-I");
           break;
   case 10: /* Print PCI */
           mdsptool(devno,"-v -p");
           break;
   case 11: /* Set PCI */
           set_register(devno,"-P");
           break;
   case 12: /* Reset */
            fl_show_alert("Function not implemented.",
            "The DSP56301 does not support a soft reset.",
            "Please use the reset button at the back of the board.",1);
            break;
   case 13: /* Write */
           mdsptool_p(devno,"-T");
           break;
   case 14: /* Read */
           mdsptool(devno,"-v -t");
           break;
   case 15: /* Dump */
           mdsptool_p(devno,"-D");
           break;
   default: break;
  }
}
