/*****************************************************************************\
*                                mdsp.h                                       *
*                                                                             *
* Contains contants and macros for the PCI mode driver mdsp and Register      *
* declarations for the Motorola DSP56xxx Board Series.                        *
\*****************************************************************************/

#ifndef _IOR
#include <sys/ioctl.h>            /* ioctl() Interface parameters    */
#endif

#define MDSP_MAX 16               /* Maximum number of DSP boards    */

#define MDSP_MAJOR 101            /* Registered Major Device Number  */

/* Documentation of Registers and Flags: See "DSP56301 Digital Signal        */
/* Processor User's Manual", Motorola 1995                                   */

/* Registers offsets of the DSP56301, add to Base memory address             */
#define HCTR 0x0010      /* Host Side Control Register  */
#define HSTR 0x0014      /* Host Side Status Register   */
#define HCVR 0x0018      /* Host Side Vector Register   */
#define HTXR 0x001c      /* Host Side Transmit Register */
#define HRXR 0x001c      /* Host Side Receive Register  (same as HTXR) */

/* Status flags for HSTR */
#define HSTR_TRDY 0x00000001 /* Bit 0: Flag Transmitter Ready          */
#define HSTR_HTRQ 0x00000002 /* Bit 1: Flag Host Transmit Data Request */
#define HSTR_HRRQ 0x00000004 /* Bit 2: Flag Host Receive Data Request  */
#define HSTR_HF3  0x00000008 /* Bit 3: Flag Host Flag 3                */
#define HSTR_HF4  0x00000010 /* Bit 4: Flag Host Flag 4                */
#define HSTR_HF5  0x00000020 /* Bit 5: Flag Host Flag 5                */
#define HSTR_HINT 0x00000040 /* Bit 6: Flag Host Interrupt A           */
#define HSTR_HREQ 0x00000080 /* Bit 7: Flag Host Request               */
/* Bits 8-31 reserved */

/* Control Flags for HCTR */
/* Bit 0: Reserved */
#define HCTR_TREQ 0x00000002 /* Bit 1:  Transmit Request Enable        */
#define HCTR_RREQ 0x00000004 /* Bit 2:  Receive  Request Enable        */
#define HCTR_HF0  0x00000008 /* Bit 3:  Host Flag 0                    */
#define HCTR_HF1  0x00000010 /* Bit 4:  Host Flag 1                    */
#define HCTR_HF2  0x00000020 /* Bit 5:  Host Flag 2                    */
#define HCTR_DMAE 0x00000040 /* Bit 6:  DMA Enable (ISA/EISA)          */
#define HCTR_SFT  0x00000080 /* Bit 7:  Prefetch/Fetch Mode            */
#define HCTR_HTF0 0x00000100 /* Bit 8:  Host->DSP   Transfer Mode 0    */
#define HCTR_HTF1 0x00000200 /* Bit 9:  Host->DSP   Transfer Mode 1    */
/* Bit 10: Reserved */
#define HCTR_HRF0 0x00000800 /* Bit 11: DSP->Host   Receive  Mode 0    */
#define HCTR_HRF1 0x00001000 /* Bit 12: DSP->Host   Receive  Mode 1    */
/* Bit 13: Reserved */
#define HCTR_HS0  0x00004000 /* Bit 14: Host Semaphore 0               */
#define HCTR_HS1  0x00008000 /* Bit 15: Host Semaphore 1               */
#define HCTR_HS2  0x00010000 /* Bit 16: Host Semaphore 2               */
/* Bits 17,18: Reserved */
#define HCTR_TWSD 0x00080000 /* Bit 19: Host Target Wait State Disable */
/* Bits 20-31: Reserved */

/*****************************************************************************\
*                       mdsp driver IOCTL Commands                            *
* param 3 is _address_ of unsigned long value to be read to                   *
* or to be written.                                                           *
\*****************************************************************************/
/* Read/Write Status Register        */
#define MDSPCTL_READ_HSTR  _IOR(MDSP_MAJOR,'s',unsigned long)
#define MDSPCTL_WRITE_HSTR _IOW(MDSP_MAJOR,'s',unsigned long)
/* Read/Write Control Register       */
#define MDSPCTL_READ_HCTR  _IOR(MDSP_MAJOR,'c',unsigned long)
#define MDSPCTL_WRITE_HCTR _IOW(MDSP_MAJOR,'c',unsigned long)
/* Read/Write Receive FIFO Register  */
#define MDSPCTL_READ_HRXR  _IOR(MDSP_MAJOR,'r',unsigned long)
#define MDSPCTL_WRITE_HRXR _IOW(MDSP_MAJOR,'r',unsigned long)
/* Read/Write Transmit FIFO Register */
#define MDSPCTL_READ_HTXR  _IOR(MDSP_MAJOR,'t',unsigned long)
#define MDSPCTL_WRITE_HTXR _IOW(MDSP_MAJOR,'t',unsigned long)
/* Read/Write Host Vector Register   */
#define MDSPCTL_READ_HCVR  _IOR(MDSP_MAJOR,'i',unsigned long)
#define MDSPCTL_WRITE_HCVR _IOW(MDSP_MAJOR,'i',unsigned long)
/* Read/Write PCI Command and Status Register */
#define MDSPCTL_READ_CCMR  _IOR(MDSP_MAJOR,'p',unsigned long)
#define MDSPCTL_WRITE_CCMR _IOW(MDSP_MAJOR,'p',unsigned long)

/* EOF mdsp.h */
